@@ -1,9 +1,10 @@
 # $File: //member/autrijus/DBIx-ReportBuilder/lib/DBIx/ReportBuilder/Render.pm $ $Author: autrijus $
-# $Revision: #3 $ $Change: 7956 $ $DateTime: 2003/09/07 23:01:03 $
+# $Revision: #8 $ $Change: 7984 $ $DateTime: 2003/09/08 17:11:41 $
 
 package DBIx::ReportBuilder::Render;
 
 use strict;
+use Safe;
 use DBIx::ReportBuilder ':all';
 use base 'XML::Twig';
 
@@ -15,10 +16,11 @@ sub new {
 	    join	=> \&join,
 	    limit	=> \&limit,
 	    orderby     => \&orderby,
+	    table	=> \&table,
+	    graph	=> \&graph,
 	    joins	=> sub { $_->delete },
 	    limits	=> sub { $_->delete },
 	    orderbys    => \&orderbys,
-	    graph	=> sub { $_->delete },
 	    %{$args{twig_handlers}||{}},
 	},
 	start_tag_handlers => {
@@ -55,7 +57,7 @@ sub loc {
 
 sub p {
     my %atts = %{$_->atts};
-    $_->del_att( DBIx::ReportBuilder->Attrs($_->tag) );
+    $_->del_att( Atts($_) );
     $_->insert(font => { face => $atts{font} }) if $atts{font};
     $_->set_att(align => $atts{align}) if $atts{align};
     my $style;
@@ -66,28 +68,60 @@ sub p {
 
 sub cells {
     my $self = shift;
-    my $SB = $_->parent->att('SearchBuilder');
-    my @children = $_->cut_children;
-    my $tr = $_->insert('thead', 'tr');
+    my $item = $_;
+    my $SB = $item->parent->att('SearchBuilder');
+    my @children = $item->cut_children;
+    my $tr = $item->insert('thead', 'tr');
     foreach my $item (@children) {
 	my $th = $tr->insert_new_elt(last_child => 'th');
 	$th->set_text($item->text);
 	$th->set_att( %{$_->atts} );
 	$item->set_tag('td');
     }
-    my $tbody = $_->insert_new_elt(last_child => 'tbody');
+    my $tbody = $item->insert_new_elt(last_child => 'tbody');
     while (my $Record = $SB->Next) {
 	my $tr = $tbody->insert_new_elt(last_child => 'tr');
+	my @fields = map { $_->att('field') } @children;
+
 	foreach my $item (@children) {
 	    my $td = $item->copy;
-	    $td->set_text($Record->{$td->att('field')});
-	    $td->del_att('field');
+	    my $text = $Record->{$td->att('field')};
+	    my $formula = $td->att('formula');
+	    if (defined($formula) and length($formula)) {
+		my $safe = Safe->new;
+		$safe->permit(qw(:base_core :base_math));
+		${$safe->varglob('_')} = $text;
+		${$safe->varglob($_)} = $Record->{$_} for @fields;
+
+		local $SIG{FPE}		= sub {};
+		local $SIG{__WARN__}	= sub {};
+		local $SIG{__DIE__}	= sub {};
+		$text = $safe->reval($formula);
+	    }
+	    $td->set_text($text);
+	    $td->del_att(qw( id field formula ));
 	    $td->paste(last_child => $tr);
 	}
     }
-    $_->parent->del_att(qw( Alias OrderBy SearchBuilder ));
-    $_->parent->del_att($self->Attrs($_->parent->tag));
-    $_->erase;
+    $item->parent->del_att(qw( Tables OrderBy SearchBuilder ));
+    $item->erase;
+}
+
+sub table {
+    my $self = shift;
+    my $item = $_;
+
+    $item->insert_new_elt('caption', {
+	map { $_ => $item->att($_) } grep { $item->att($_) } qw( font size )
+    }, $item->att('caption') );
+    $item->del_att(Atts($item));
+}
+
+sub graph {
+    my $self = shift;
+    my $item = $_;
+    $item->set_tag('table');
+    return $self->table; # XXX
 }
 
 sub search {
@@ -103,22 +137,27 @@ sub search {
 	$SB->FirstRow( $item )
     }
 
-    $_->set_att('Alias' => {});
+    $_->set_att('Tables' => {});	    # key: table, val: alias
     $_->set_att('OrderBy' => []);
     $_->set_att('SearchBuilder' => $SB);
 }
 
 sub _alias {
-    my $Alias = $_[0]->parent->parent->att('Alias');
-    my $rv = $Alias->{$_[0]->att('alias')} or return;
+    my $Tables = $_[0]->parent->parent->att('Tables');
+    my $rv = $Tables->{$_[0]->att('table')} or return;
     return (ALIAS => $rv);
 };
 
 sub join {
     my $item  = $_;
-    my $SB    = $item->parent->parent->att('SearchBuilder');
-    my $Alias = $item->parent->parent->att('Alias');
-    $Alias->{$_->att('alias')} = $SB->Join(
+    my $SB     = $item->parent->parent->att('SearchBuilder');
+    my $Tables = $item->parent->parent->att('Tables');
+    $Tables->{$_->att('table2')} = $SB->Join(
+	TYPE	=> ($item->att('type') || 'left'),
+	ALIAS1	=> ($Tables->{$item->att('table') || ''} || 'main'),
+	FIELD1	=> $item->att('field'),
+	TABLE2	=> $item->att('table2'),
+	FIELD2	=> $item->att('field2'),
 	map { uc($_) => $item->att($_) } $item->att_names
     );
 }
